#!/usr/bin/env python
# coding: utf-8

# In[11]:


def generate_selections(items, n):
    '''Generate selections of n items.'''
    # Make a list to tell whether an item is in the current selection.
    in_selection = [False for i in range(len(items))]

    # Make a result list.
    results = []

    # Build the combinations recursively.
    select_items(items, in_selection, results, n, 0)

    # Return the results.
    return results


# In[12]:


def select_items(items, in_selection, results, n, first_item):
    '''
    Recursively select n additional items with indexes >= first_item.
    If n == 0, add the current combination to the results.
    '''
    if n == 0:
        # Add the current selection to the results.
        selection = []
        for i in range(len(items)):
            # If this item is selected, add it to the selection.
            if in_selection[i]:
                selection.append(items[i])
        results.append(selection)
    else:
        # Try adding each of the remaining items.
        for i in range(first_item, len(items)):
            # Try adding this item.
            in_selection[i] = True

            # Recursively add the rest of the required items.
            select_items(items, in_selection, results, n - 1, i + 1)

            # Remove this item from the selection.
            in_selection[i] = False


# In[26]:


import math

items = ['Ant', 'Bat', 'Cat', 'Dog', 'Eel']
k = 3
result = generate_selections(items, k)
assert len(result) == math.comb(len(items), k)
print(f'{len(result)} selections')
for selection in result:
    print(selection)
print()

items = [i for i in range(7)]
k = 4
result = generate_selections(items, k)
assert len(result) == math.comb(len(items), k)
print(f'{len(result)} selections')
for selection in result:
    print(selection)

